searchData={"items":[{"type":"module","title":"Mint.HTTP","doc":"Process-less HTTP connection data structure and functions.\n\nSingle interface for `Mint.HTTP1` and `Mint.HTTP2` with support for version\nnegotiation and proxies.","ref":"Mint.HTTP.html"},{"type":"module","title":"Usage - Mint.HTTP","doc":"To establish a connection with a given server, use `connect/4`. This will\nreturn an opaque data structure that represents the connection\nto the server. To send a request, you can use `request/5`. Sending a request\ndoes not take care of the response to that request, instead we use `Mint.HTTP.stream/2`\nto process the response, which we will look at in just a bit. The connection is a\nwrapper around a TCP (`:gen_tcp` module) or SSL (`:ssl` module) socket that is\nset in **active mode** (with `active: :once`). This means that TCP/SSL messages\nwill be delivered to the process that started the connection.\n\nThe process that owns the connection is responsible for receiving the messages\n(for example, a GenServer is responsible for defining `handle_info/2`). However,\n`Mint.HTTP` makes it easy to identify TCP/SSL messages that are coming from the\nconnection with the server with the `stream/2` function. This function takes the\nconnection and a term and returns `:unknown` if the term is not a TCP/SSL message\nbelonging to the connection. If the term *is* a message for the connection, then\na response and a new connection are returned. It's important to store the new\nreturned connection data structure over the old one since the connection is an\nimmutable data structure.\n\nLet's see an example of a common workflow of connecting to a server, sending a\nrequest, and processing the response. We start by using `connect/3` to connect\nto a server.\n\n    {:ok, conn} = Mint.HTTP.connect(:http, \"httpbin.org\", 80)\n\n`conn` is a data structure that represents the connection.\n\nTo send a request, we use `request/5`.\n\n    {:ok, conn, request_ref} = Mint.HTTP.request(conn, \"GET\", \"/\", [], nil)\n\nAs you can see, sending a request returns a new updated `conn` struct and a\n`request_ref`. The updated connection struct is returned because the connection\nis an immutable structure keeping the connection state, so every action we do on it must return a new,\npossibly updated, connection that we're responsible for storing over the old\none. `request_ref` is a unique reference that can be used to identify which\nrequest a given response belongs to.\n\nNow that we sent our request, we're responsible for receiving the messages that\nthe TCP/SSL socket will send to our process. For example, in a GenServer\nwe would do that with a `handle_info/2` callback. In our case, we're going to\nuse a simple `receive`. `Mint.HTTP` provides a way to tell if a message comes\nfrom the socket wrapped by our connection or not: the `stream/2` function. If\nthe message we pass to it is not destined for our connection, this function returns\n`:unknown`. Otherwise, it returns an updated connection and one or more responses.\n\n    receive do\n      message ->\n        case Mint.HTTP.stream(conn, message) do\n          :unknown -> handle_normal_message(message)\n          {:ok, conn, responses} -> handle_responses(conn, responses)\n        end\n    end\n\n`responses` is a list of possible responses. The most common responses are:\n\n  * `{:status, request_ref, status_code}` for the status code\n  * `{:headers, request_ref, headers}` for the response headers\n  * `{:data, request_ref, binary}` for pieces of the response body\n  * `{:done, request_ref}` for the end of the response\n\nAs you can see, all responses have the unique request reference as the second\nelement of the tuple, so that we know which request the response belongs to.\nSee `t:Mint.Types.response/0` for the full list of responses returned by `Mint.HTTP.stream/2`.","ref":"Mint.HTTP.html#module-usage"},{"type":"module","title":"Architecture - Mint.HTTP","doc":"A processless architecture like the one here requires a few modifications to how\nwe use this HTTP client. Usually, you will want to create this data structure\nin a process that acts as *connection manager*. Sometimes, you might want to\nhave a single process responsible for multiple connections, either to just one\nhost or multiple hosts. For more discussion on architectures based off of this\nHTTP client, see the [*Architecture*](architecture.html) page in the docs.","ref":"Mint.HTTP.html#module-architecture"},{"type":"module","title":"SSL certificates - Mint.HTTP","doc":"When using SSL, you can pass in your own CA certificate store or use one provided by Mint. Mint\ndoesn't ship with the certificate store itself, but it has an optional dependency on\n[CAStore](https://github.com/elixir-mint/castore), which provides an up-to-date certificate store. If\nyou don't want to use your own certificate store, just add `:castore` to your dependencies.\n\nStarting [from OTP\n25](https://www.erlang.org/blog/my-otp-25-highlights/#ca-certificates-can-be-fetched-from-the-os-standard-place),\nyou can also load certificates from a file\n([`:public_key.cacerts_load/1`](https://www.erlang.org/doc/man/public_key.html#cacerts_load-1)).\nYou can also get certificate from the OS trust store using\n[`:public_key.cacerts_get/0`](https://www.erlang.org/doc/man/public_key.html#cacerts_get-0).\nIf you are using OTP 25+ it is recommended to set this option.\n\n    Mint.HTTP.connect(:https, host, port, transport_opts: [cacerts: :public_key.cacerts_get()])","ref":"Mint.HTTP.html#module-ssl-certificates"},{"type":"module","title":"Mode - Mint.HTTP","doc":"By default Mint operates in **active mode** meaning that the process that started the\nconnection receives socket messages. Mint also supports **passive mode**, where no messages\nare sent to the process and the process needs to fetch data out of the socket manually.\nThe mode can be controlled at connection time through the `:mode` option in `connect/4`\nor changed dynamically through `set_mode/2`. Passive mode is generally only recommended\nfor special use cases.","ref":"Mint.HTTP.html#module-mode"},{"type":"module","title":"Logging - Mint.HTTP","doc":"Mint uses the `Logger` module to log information about the connection. Most logs are\nemitted *since version 1.5.0*. The logs are not emitted by default, since we consider\nMint to be too low level. However, you can enable logging by passing `log: true` to\n`connect/4`.\n\n> #### Changes to the Format of Logs {: .warning}\n>\n> The format of logs emitted by Mint might change without notice between any versions,\n> without it being considered a breaking change. You are only meant to control what\n> gets logged by using the `Logger` API and Erlang's `:logger` module.","ref":"Mint.HTTP.html#module-logging"},{"type":"function","title":"Mint.HTTP.close/1","doc":"Closes the given connection.\n\nThis function closes the socket wrapped by the given connection. Once the socket\nis closed, the connection goes into the \"closed\" state and `open?/1` returns `false`.\nYou can throw away a closed connection.\n\nClosing a connection does not guarantee that data that is in flight gets delivered\nto the server.\n\nAlways returns `{:ok, conn}` where `conn` is the updated connection.","ref":"Mint.HTTP.html#close/1"},{"type":"function","title":"Examples - Mint.HTTP.close/1","doc":"{:ok, conn} = Mint.HTTP.close(conn)","ref":"Mint.HTTP.html#close/1-examples"},{"type":"function","title":"Mint.HTTP.connect/4","doc":"Creates a new connection to a given server.\n\nCreates a new connection struct and establishes the connection to the given server,\nidentified by the given `host` and `port` combination. Both HTTP and HTTPS are supported\nby passing respectively `:http` and `:https` as the `scheme`.\n\nThe connection struct wraps a socket, which is created once the connection\nis established inside this function. If HTTP is used, then the created socket is a TCP\nsocket and the `:gen_tcp` module is used to create that socket. If HTTPS is used, then\nthe created socket is an SSL socket and the `:ssl` module is used to create that socket.\nThe socket is created in active mode (with `active: :once`), which is why it is important\nto know the type of the socket: messages from the socket will be delivered directly to the\nprocess that creates the connection and tagged appropriately by the socket module (see the\n`:gen_tcp` and `:ssl` modules). See `stream/2` for more information on the messages and\nhow to process them and on the socket mode.","ref":"Mint.HTTP.html#connect/4"},{"type":"function","title":"Options - Mint.HTTP.connect/4","doc":"* `:hostname` - (string) explicitly provide the hostname used for the `Host` header,\n    hostname verification, SNI, and so on. **Required when `address` is not a string.**\n\n  * `:transport_opts` - (keyword) options to be given to the transport being used.\n    These options will be merged with some default options that cannot be overridden.\n    For more details, refer to the \"Transport options\" section below.\n\n  * `:mode` - (`:active` or `:passive`) whether to set the socket to active or\n    passive mode. See the \"Mode\" section in the module documentation and `set_mode/2`.\n\n  * `:protocols` - (list of atoms) a list of protocols to try when connecting to the\n    server. The possible values in the list are `:http1` for HTTP/1 and HTTP/1.1 and\n    `:http2` for HTTP/2. If only one protocol is present in the list, then the connection\n    will be forced to use that protocol. If both `:http1` and `:http2` are present in the\n    list, then Mint will negotiate the protocol. See the section \"Protocol negotiation\"\n    below for more information. Defaults to `[:http1, :http2]`.\n\n  * `:proxy_headers` - a list of headers (`t:Mint.Types.headers/0`) to pass when using\n    a proxy. They will be used for the `CONNECT` request in tunnel proxies or merged\n    with every request for forward proxies.\n\n  * `:log` - (boolean) whether this connection logs or not. See the [\"Logging\"\n    section](#module-logging) in the module documentation. Defaults to `false`.\n    *Available since v1.5.0*.\n\nThe following options are HTTP/1-specific and will force the connection\nto be an HTTP/1 connection.\n\n  * `:proxy` - a `{scheme, address, port, opts}` tuple that identifies a proxy to\n    connect to. See the \"Proxying\" section below for more information.\n\nThe following options are HTTP/2-specific and will only be used on HTTP/2 connections.\n\n  * `:client_settings` - (keyword) a list of client HTTP/2 settings to send to the\n    server. See `Mint.HTTP2.put_settings/2` for more information. This is only used\n    in HTTP/2 connections.\n\nThere may be further protocol specific options that only take effect when the corresponding\nconnection is established. Check `Mint.HTTP1.connect/4` and `Mint.HTTP2.connect/4` for\ndetails.","ref":"Mint.HTTP.html#connect/4-options"},{"type":"function","title":"Protocol negotiation - Mint.HTTP.connect/4","doc":"If both `:http1` and `:http2` are present in the list passed in the `:protocols` option,\nthe protocol negotiation happens in the following way:\n\n  * If the scheme used to connect to the server is `:http`, then HTTP/1 or HTTP/1.1 is used.\n\n  * If the scheme is `:https`, then ALPN negotiation is used to determine the right\n    protocol. This means that the server will decide whether to use HTTP/1 or\n    HTTP/2. If the server doesn't support protocol negotiation, we will fall back to\n    HTTP/1. If the server negotiates a protocol that we don't know how to handle,\n    `{:error, {:bad_alpn_protocol, protocol}}` is returned.","ref":"Mint.HTTP.html#connect/4-protocol-negotiation"},{"type":"function","title":"Proxying - Mint.HTTP.connect/4","doc":"You can set up proxying through the `:proxy` option, which is a tuple\n`{scheme, address, port, opts}` that identifies the proxy to connect to.\nOnce a proxied connection is returned, the proxy is transparent to you and you\ncan use the connection like a normal HTTP/1 connection.\n\nIf the `scheme` is `:http`, we will connect to the host in the most compatible\nway, supporting older proxy servers. Data will be sent in clear text.\n\nIf the connection scheme is `:https`, we will connect to the host with a tunnel\nthrough the proxy. Using `:https` for both the proxy and the connection scheme\nis not supported, it is recommended to use `:https` for the end host connection\ninstead of the proxy.","ref":"Mint.HTTP.html#connect/4-proxying"},{"type":"function","title":"Transport options - Mint.HTTP.connect/4","doc":"The options specified in `:transport_opts` are passed to the module that\nimplements the socket interface: `:gen_tcp` when the scheme is `:http`, and\n`:ssl` when the scheme is `:https`. Please refer to the documentation for those\nmodules, as well as for `:inet.setopts/2`, for a detailed description of all\navailable options.\n\nThe behaviour of some options is modified by Mint, as described below.\n\nA special case is the `:timeout` option, which is passed to the transport\nmodule's `connect` function to limit the amount of time to wait for the\nnetwork connection to be established.\n\nCommon options for `:http` and `:https`:\n\n  * `:active` - controlled by the `:mode` option. Cannot be overridden.\n\n  * `:mode` - set to `:binary`. Cannot be overridden.\n\n  * `:packet` - set to `:raw`. Cannot be overridden.\n\n  * `:timeout` - connect timeout in milliseconds. Defaults to `30_000` (30\n    seconds), and may be overridden by the caller. Set to `:infinity` to\n    disable the connect timeout.\n\n  * `:inet6` - if set to `true` enables IPv6 connection. Defaults to `false`\n    and may be overridden by the caller.\n\n  * `:inet4` - if set to `true` falls back to IPv4 if IPv6 connection fails.\n    Defaults to `true` and may be overridden by the caller. *Available since\n    v1.6.0*.\n\nOptions for `:https` only:\n\n  * `:alpn_advertised_protocols` - managed by Mint. Cannot be overridden.\n\n  * `:cacerts` - certificates of types `:ssl.client_cacerts()`.\n    If `:verify` is set to `:verify_peer` (the default) and\n    no CA trust store is specified using the `:cacertfile` or `:cacerts`\n    option, Mint will attempt to use the trust store from the\n    [CAStore](https://github.com/elixir-mint/castore) package or raise an\n    exception if this package is not available. It is recommended to set this\n    option to `:public_key.cacerts_get()`.\n\n  * `:cacertfile` - path to a file containing PEM-encoded CA certificates.\n    See the `:cacerts` option for the defaults to this value.\n\n  * `:ciphers` - defaults to the lists returned by\n    `:ssl.filter_cipher_suites(:ssl.cipher_suites(:all, version), [])`\n    where `version` is each value in the `:versions` setting. This list is\n    then filtered according to the blocklist in\n    [RFC7540 appendix A](https://tools.ietf.org/html/rfc7540#appendix-A);\n    May be overridden by the caller. See the \"Supporting older cipher suites\"\n    section below for some examples.\n\n  * `:depth` - defaults to `4`. May be overridden by the caller.\n\n  * `:partial_chain` - unless a custom `:partial_chain` function is specified,\n    Mint will enable its own partial chain handler, which accepts server\n    certificate chains containing a certificate that was issued by a\n    CA certificate in the CA trust store, even if that certificate is not\n    last in the chain. This improves interoperability with some servers\n    (for example, with a cross-signed intermediate CA or some misconfigured servers),\n    but is a less strict interpretation of the TLS specification than the\n    Erlang/OTP default behaviour.\n\n  * `:reuse_sessions` - defaults to `true`. May be overridden by the caller. If\n    `:\"tlsv1.3\"` is the only TLS version specified, `:reuse_sessions` will be\n    removed from the options.\n\n  * `:secure_renegotiate` - defaults to `true`. May be overridden by the\n    caller. If `:\"tlsv1.3\"` is the only TLS version specified, `:secure_renegotiate`\n    will be removed from the options.\n\n  * `:server_name_indication` - defaults to specified destination hostname.\n    May be overridden by the caller.\n\n  * `:verify` - defaults to `:verify_peer`. May be overridden by the caller.\n\n  * `:verify_fun` - unless a custom `:verify_fun` is specified, or `:verify`\n    is set to `:verify_none`, Mint will enable hostname verification with\n    support for wildcards in the server's 'SubjectAltName' extension, similar\n    to the behaviour implemented in\n    `:public_key.pkix_verify_hostname_match_fun(:https)` in recent Erlang/OTP\n    releases. This improves compatibility with recently issued wildcard\n    certificates also on older Erlang/OTP releases.\n\n  * `:versions` - defaults to `[:\"tlsv1.2\"]` (TLS v1.2 only). May be\n    overridden by the caller.","ref":"Mint.HTTP.html#connect/4-transport-options"},{"type":"function","title":"Supporting older cipher suites - Mint.HTTP.connect/4","doc":"By default only a small list of modern cipher suites is enabled, in compliance\nwith the HTTP/2 specification. Some servers, in particular HTTP/1 servers, may\nnot support any of these cipher suites, resulting in TLS handshake failures or\nclosed connections.\n\nTo select the default cipher suites of Erlang/OTP (including for example\nAES-CBC), use the following `:transport_opts`:\n\n    # Erlang/OTP 20.3 or later:\n    transport_opts: [ciphers: :ssl.cipher_suites(:default, :\"tlsv1.2\")]\n    # Older versions:\n    transport_opts: [ciphers: :ssl.cipher_suites()]\n\nRecent Erlang/OTP releases do not enable RSA key exchange by default, due to\nknown weaknesses. If necessary, you can build a cipher list with RSA exchange\nand use it in `:transport_opts`:\n\n    ciphers =\n      :ssl.cipher_suites(:all, :\"tlsv1.2\")\n      |> :ssl.filter_cipher_suites(\n        key_exchange: &(&1 == :rsa),\n        cipher: &(&1 in [:aes_256_gcm, :aes_128_gcm, :aes_256_cbc, :aes_128_cbc])\n      )\n      |> :ssl.append_cipher_suites(:ssl.cipher_suites(:default, :\"tlsv1.2\"))","ref":"Mint.HTTP.html#connect/4-supporting-older-cipher-suites"},{"type":"function","title":"Examples - Mint.HTTP.connect/4","doc":"{:ok, conn} = Mint.HTTP.connect(:http, \"httpbin.org\", 80)\n\nUsing a proxy:\n\n    proxy = {:http, \"myproxy.example.com\", 80, []}\n    {:ok, conn} = Mint.HTTP.connect(:https, \"httpbin.org\", 443, proxy: proxy)\n\nForcing the connection to be an HTTP/2 connection:\n\n    {:ok, conn} = Mint.HTTP.connect(:https, \"httpbin.org\", 443, protocols: [:http2])\n\nEnable all default cipher suites of Erlang/OTP (release 20.3 or later):\n\n    opts = [transport_opts: [ciphers: :ssl.cipher_suites(:default, :\"tlsv1.2\")]]\n    {:ok, conn} = Mint.HTTP.connect(:https, \"httpbin.org\", 443, opts)","ref":"Mint.HTTP.html#connect/4-examples"},{"type":"function","title":"Mint.HTTP.controlling_process/2","doc":"Changes the *controlling process* of the given connection to `new_pid`.\n\nThe **controlling process** is a concept that comes from the Erlang TCP and\nSSL implementations. The controlling process of a connection is the process\nthat started the connection and that receives the messages for that connection.\nYou can change the controlling process of a connection through this function.\n\nThis function also takes care of \"transferring\" all the connection messages\nthat are in the mailbox of the current controlling process to the new\ncontrolling process.\n\nRemember that the connection is a data structure, so if you\nchange the controlling process it doesn't mean you \"transferred\" the\nconnection data structure itself to the other process, which you have\nto do manually (for example by sending the connection data structure to the\nnew controlling process). If you do that, be careful of race conditions\nand be sure to retrieve the connection in the new controlling process\nbefore accepting connection messages in the new controlling process.\nIn fact, this function is guaranteed to return the connection unchanged,\nso you are free to ignore the connection entry returned in `{:ok, conn}`.","ref":"Mint.HTTP.html#controlling_process/2"},{"type":"function","title":"Examples - Mint.HTTP.controlling_process/2","doc":"send(new_pid, {:conn, conn})\n    {:ok, conn} = Mint.HTTP.controlling_process(conn, new_pid)\n\n    # In the \"new_pid\" process\n    receive do\n      {:conn, conn} ->\n        # Will receive connection messages.\n    end","ref":"Mint.HTTP.html#controlling_process/2-examples"},{"type":"function","title":"Mint.HTTP.delete_private/2","doc":"Deletes a value in the private store.\n\nDeletes the private value stored under `key` in the connection. Returns the\nupdated connection.\n\nSee also `put_private/3` and `get_private/3`.","ref":"Mint.HTTP.html#delete_private/2"},{"type":"function","title":"Examples - Mint.HTTP.delete_private/2","doc":"conn = Mint.HTTP.put_private(conn, :client_name, \"Mint\")\n\n    Mint.HTTP.get_private(conn, :client_name)\n    #=> \"Mint\"\n\n    conn = Mint.HTTP.delete_private(conn, :client_name)\n    Mint.HTTP.get_private(conn, :client_name)\n    #=> nil","ref":"Mint.HTTP.html#delete_private/2-examples"},{"type":"function","title":"Mint.HTTP.get_private/3","doc":"Gets a private value from the connection.\n\nRetrieves a private value previously set with `put_private/3` from the connection.\n`key` is the key under which the value to retrieve is stored. `default` is a default\nvalue returned in case there's no value under the given key.\n\nSee also `put_private/3` and `delete_private/2`.","ref":"Mint.HTTP.html#get_private/3"},{"type":"function","title":"Examples - Mint.HTTP.get_private/3","doc":"conn = Mint.HTTP.put_private(conn, :client_name, \"Mint\")\n\n    Mint.HTTP.get_private(conn, :client_name)\n    #=> \"Mint\"\n\n    Mint.HTTP.get_private(conn, :non_existent)\n    #=> nil","ref":"Mint.HTTP.html#get_private/3-examples"},{"type":"function","title":"Mint.HTTP.get_proxy_headers/1","doc":"Gets the proxy headers associated with the connection in the `CONNECT` method.\n\nWhen using tunnel proxy and HTTPs, the only way to exchange data with\nthe proxy is through headers in the `CONNECT` method.","ref":"Mint.HTTP.html#get_proxy_headers/1"},{"type":"function","title":"Mint.HTTP.get_socket/1","doc":"Gets the socket associated with the connection.\n\nDo not use the returned socket to change its internal state. Only read information from the socket.\nFor instance, use `:ssl.connection_information/2` to retrieve TLS-specific information from the\nsocket.","ref":"Mint.HTTP.html#get_socket/1"},{"type":"macro","title":"Mint.HTTP.is_connection_message/2","doc":"Macro to check that a given received `message` is intended for the given connection `conn`.\n\nThis guard is useful in `receive` loops or in callbacks that handle generic messages (such as a\n`c:GenServer.handle_info/2` callback) so that you don't have to hand the `message` to\n`Mint.HTTP.stream/2` and check for the `:unknown_message` return value.\n\nThis macro can be used in guards.\n\n**Note**: this macro is only available if you compile Mint with Elixir 1.10.0 or greater (and\nOTP 21+, which is required by Elixir 1.10.0 and on).","ref":"Mint.HTTP.html#is_connection_message/2"},{"type":"macro","title":"Examples - Mint.HTTP.is_connection_message/2","doc":"require Mint.HTTP\n\n    {:ok, conn, request_ref} = Mint.HTTP.request(conn, \"POST\", \"/\", headers, \"\")\n\n    receive do\n      message when Mint.HTTP.is_connection_message(conn, message) ->\n        Mint.HTTP.stream(conn, message)\n\n      other ->\n        # This message is related to something else or to some other connection\n    end","ref":"Mint.HTTP.html#is_connection_message/2-examples"},{"type":"function","title":"Mint.HTTP.open?/2","doc":"Checks whether the connection is open.\n\nThis function returns `true` if the connection is open for the given `type`,\n`false` otherwise. It should be used to check that a connection is open before\nsending requests or performing operations that involve talking to the server.\n\nThe `type` argument can be used to tell whether the connection is open for both reading\nand writing, only open for reading, or closed for both. In HTTP/1, a connection is always\neither open, or closed (for both reading and writing). In HTTP/2, the connection can be closed only\nfor writing but not for reading, meaning that you cannot send any more data to the\nserver but you can still receive data from the server. In this case, `Mint.HTTP.open?(conn, :read)`\nwould return `true` but `Mint.HTTP.open?(conn, :write)` would return `false`.\nSee the \"Closed connection\" section in the module documentation of `Mint.HTTP2`.\n\nIf a connection is *completely closed* (that is, `Mint.HTTP.open?(conn, :read)` returns `false`),\nit has become useless and you should get rid of it. If you still need a connection\nto the server, start a new connection with `connect/4`.\n\n> #### The default value of `type` is `:write` {: .warning}\n>\n> With the default value of `type` being `:write`, a call to\n> `Mint.HTTP.open?(conn)` will return `false` if `conn` was closed for writing\n> but is still open for reading. If you need to make sure the connection is\n> completely closed, check that `Mint.HTTP.open?(conn, :read)` returns `false`.","ref":"Mint.HTTP.html#open?/2"},{"type":"function","title":"Examples - Mint.HTTP.open?/2","doc":"{:ok, conn} = Mint.HTTP.connect(:http, \"httpbin.org\", 80)\n    Mint.HTTP.open?(conn)\n    #=> true","ref":"Mint.HTTP.html#open?/2-examples"},{"type":"function","title":"Mint.HTTP.open_request_count/1","doc":"Returns the number of open requests.\n\nOpen requests are requests that have not yet received a `:done` response.\nThis function returns the number of open requests for both HTTP/1 and HTTP/2,\nbut for HTTP/2 only client-initiated requests are considered as open requests.\nSee `Mint.HTTP2.open_request_count/1` for more information.","ref":"Mint.HTTP.html#open_request_count/1"},{"type":"function","title":"Examples - Mint.HTTP.open_request_count/1","doc":"{:ok, conn, _ref} = Mint.HTTP.request(conn, \"GET\", \"/\", [])\n    Mint.HTTP.open_request_count(conn)\n    #=> 1","ref":"Mint.HTTP.html#open_request_count/1-examples"},{"type":"function","title":"Mint.HTTP.protocol/1","doc":"Returns the protocol used by the current connection.","ref":"Mint.HTTP.html#protocol/1"},{"type":"function","title":"Examples - Mint.HTTP.protocol/1","doc":"iex> Mint.HTTP.protocol(%Mint.HTTP1{})\n    :http1\n\n    iex> Mint.HTTP.protocol(%Mint.HTTP2{})\n    :http2","ref":"Mint.HTTP.html#protocol/1-examples"},{"type":"function","title":"Mint.HTTP.put_log/2","doc":"Sets whether the connection should log information or not.\n\nSee the [\"Logging\" section](#module-logging) in the module documentation for more information.","ref":"Mint.HTTP.html#put_log/2"},{"type":"function","title":"Mint.HTTP.put_private/3","doc":"Assigns a new private key and value in the connection.\n\nThis storage is meant to be used to associate metadata with the connection and\nit can be useful when handling multiple connections.\n\nThe given `key` must be an atom, while the given `value` can be an arbitrary\nterm. The return value of this function is an updated connection.\n\nSee also `get_private/3` and `delete_private/2`.","ref":"Mint.HTTP.html#put_private/3"},{"type":"function","title":"Examples - Mint.HTTP.put_private/3","doc":"Let's see an example of putting a value and then getting it:\n\n    conn = Mint.HTTP.put_private(conn, :client_name, \"Mint\")\n    Mint.HTTP.get_private(conn, :client_name)\n    #=> \"Mint\"","ref":"Mint.HTTP.html#put_private/3-examples"},{"type":"function","title":"Mint.HTTP.recv/3","doc":"Receives data from the socket in a blocking way.\n\nBy default Mint operates in active mode, meaning that messages are delivered\nto the process that started the connection. However, Mint also supports passive\nmode (see the \"Mode\" section in the module documentation).\n\nIn passive mode, you'll need to manually get bytes out of the socket. You can\ndo that with this function.\n\n`byte_count` is the number of bytes you want out of the socket. If `byte_count`\nis `0`, all available bytes will be returned.\n\n`timeout` is the maximum time to wait before returning an error.\n\nThis function will raise an error if the socket is in active mode.\n\n> #### Hanging Waiting for Bytes {: .warning}\n>\n> If `byte_count` is greater than `0` and the socket doesn't receive\n> *at least* `byte_count` bytes withing the `timeout`, then the function\n> will block for the duration of `timeout` and then return a timeout error.\n> This behavior is the same as the `recv` function in [`:gen_tcp`](`:gen_tcp`)\n> and [`:ssl`](`:ssl`).","ref":"Mint.HTTP.html#recv/3"},{"type":"function","title":"Examples - Mint.HTTP.recv/3","doc":"{:ok, conn, responses} = Mint.HTTP.recv(conn, 0, 5000)","ref":"Mint.HTTP.html#recv/3-examples"},{"type":"function","title":"Mint.HTTP.request/5","doc":"Sends a request to the connected server.\n\nThis function sends a new request to the server that `conn` is connected to.\n`method` is a string representing the method for the request, such as `\"GET\"`\nor `\"POST\"`. `path` is the path on the host to send the request to. `headers`\nis a list of request headers in the form `{header_name, header_value}` with\n`header_name` and `header_value` being strings. `body` can have one of three\nvalues:\n\n  * `nil` - no body is sent with the request.\n\n  * iodata - the body to send for the request.\n\n  * `:stream` - when the value of the body is `:stream` the request\n    body can be streamed on the connection. See `stream_request_body/3`.\n    In HTTP/1, you can't open a request if the body of another request is\n    streaming.\n\nIf the request is sent correctly, this function returns `{:ok, conn, request_ref}`.\n`conn` is an updated connection that should be stored over the old connection.\n`request_ref` is a unique reference that can be used to match on responses for this\nrequest that are returned by `stream/2`. See `stream/2` for more information.\n\nIf there's an error with sending the request, `{:error, conn, reason}` is returned.\n`reason` is the cause of the error. `conn` is an updated connection. It's important\nto store the returned connection over the old connection in case of errors too, because\nthe state of the connection might change when there are errors as well. An error when\nsending a request **does not** necessarily mean that the connection is closed. Use\n`open?/1` to verify that the connection is open.\n\nRequests can be pipelined so the full response does not have to received\nbefore the next request can be sent. It is up to users to verify that the\nserver supports pipelining and that the request is safe to pipeline.\n\nIn HTTP/1, you can't open a request if the body of another request is streaming.\nSee `Mint.HTTP1.request/5` for more information.\n\nFor a quick discussion on HTTP/2 streams and requests, see the `Mint.HTTP2` module and\n`Mint.HTTP2.request/5`.","ref":"Mint.HTTP.html#request/5"},{"type":"function","title":"The `content-length` header - Mint.HTTP.request/5","doc":"If you don't set the `content-length` header and you send a body with the request (that\nis, not `nil` and not `:stream`), then Mint will add a default `content-length` header\nto your request. If you're using HTTP/2 and streaming the request, you may provide the\n`content-length` header yourself. If you're using HTTP/1, Mint will do chunked\ntransfer-encoding when a content-length is not provided (see `Mint.HTTP1.request/5`).","ref":"Mint.HTTP.html#request/5-the-content-length-header"},{"type":"function","title":"Examples - Mint.HTTP.request/5","doc":"Mint.HTTP.request(conn, \"GET\", \"/\", _headers = [], _body = nil)\n    Mint.HTTP.request(conn, \"POST\", \"/path\", [{\"content-type\", \"application/json\"}], \"{}\")","ref":"Mint.HTTP.html#request/5-examples"},{"type":"function","title":"Mint.HTTP.set_mode/2","doc":"Changes the mode of the underlying socket.\n\nTo use the connection in *active mode*, where the process that started the\nconnection receives socket messages, set the mode to `:active` (see also `stream/2`).\nTo use the connection in *passive mode*, where you need to manually receive data\nfrom the socket, set the mode to `:passive` (see also `recv/3`).\n\nThe mode can also be controlled at connection time by the `:mode` option passed\nto `connect/4`.\n\nNote that if you're switching from active to passive mode, you still might have\nsocket messages in the process mailbox that you need to consume before doing\nany other operation on the connection.\n\nSee the \"Mode\" section in the module documentation for more information on modes.","ref":"Mint.HTTP.html#set_mode/2"},{"type":"function","title":"Examples - Mint.HTTP.set_mode/2","doc":"{:ok, conn} = Mint.HTTP.set_mode(conn, :passive)","ref":"Mint.HTTP.html#set_mode/2-examples"},{"type":"function","title":"Mint.HTTP.stream/2","doc":"Streams the next batch of responses from the given `message`.\n\nThis function processes a \"message\" which can be any term, but should be\na message received by the process that owns the connection. **Processing**\na message means that this function will parse it and check if it's a message\nthat is directed to this connection, that is, a TCP/SSL message received on the\nconnection's socket. If it is, then this function will parse the message,\nturn it into a list of responses, and possibly take action given the responses.\nAs an example of an action that this function could perform, if the server sends\na ping request this function will transparently take care of pinging the server back.\n\nIf there's no error, this function returns `{:ok, conn, responses}` where `conn` is\nthe updated connection and `responses` is a list of responses. See the \"Responses\"\nsection below. If there's an error, `{:error, conn, reason, responses}` is returned,\nwhere `conn` is the updated connection, `reason` is the error reason, and `responses`\nis a list of responses that were correctly parsed before the error.\n\n> #### Graceful Close {: .tip}\n>\n> If this function returns `{:ok, conn, responses}`, it doesn't *necessarily* mean\n> that the connection is still open. For example, TCP/SSL **close** messages are treated\n> as errors only if there are in-flight requests. If there are no in-flight requests,\n> the connection is closed gracefully and `{:ok, conn, responses}` is returned.\n> Always check with `open?/1` to see if the connection is still open.\n\nIf the given `message` is not from the connection's socket,\nthis function returns `:unknown`.\n\n> #### Receiving Multiple Messages {: .warning}\n>\n> Your connection and the HTTP server can exchange multiple **protocol-specific messages**\n> on the socket that don't necessarily *produce responses*. For example, the HTTP server\n> might tell the connection to update some internal settings. For this reason, you\n> should always receive as many messages coming to your process as possible, for example\n> by using `receive` recursively. You can see an example of this approach in the\n> [\"Usage Examples\" documentation](architecture.html#usage-examples).","ref":"Mint.HTTP.html#stream/2"},{"type":"function","title":"Socket mode - Mint.HTTP.stream/2","doc":"Mint sets the socket in `active: :once` mode. This means that a single socket\nmessage at a time is delivered to the process that owns the connection. After\na message is delivered, then no other messages are delivered (we say the socket\ngoes in *passive* mode). When `stream/2` is called to process the message that\nwas received, Mint sets the socket back to `active: :once`. This is good to know\nin order to understand how the socket is handled by Mint, but in normal usage\nit just means that you will process one message at a time with `stream/2` and not\npay too much attention to the socket mode.\n\nMint also supports passive mode to avoid receiving messages. See the \"Mode\" section\nin the module documentation.","ref":"Mint.HTTP.html#stream/2-socket-mode"},{"type":"function","title":"Responses - Mint.HTTP.stream/2","doc":"Each possible response returned by this function is a tuple with two or more elements.\nThe first element is always an atom that identifies the kind of response. The second\nelement is a unique reference `t:Mint.Types.request_ref/0` that identifies the request\nthat the response belongs to. This is the term returned by `request/5`. After these\ntwo elements, there can be response-specific terms as well, documented below.\n\nThese are the possible responses that can be returned.\n\n  * `{:status, request_ref, status_code}` - returned when the server replied\n    with a response status code. The status code is a non-negative integer.\n    You can have zero or more `1xx` `:status` and `:headers` responses for a\n    single request, but they all precede a single non-`1xx` `:status` response.\n\n  * `{:headers, request_ref, headers}` - returned when the server replied\n    with a list of headers. Headers are in the form `{header_name, header_value}`\n    with `header_name` and `header_value` being strings. A single `:headers` response\n    will come after the `:status` response. A single `:headers` response may come\n    after all the `:data` responses if **trailer headers** are present.\n\n  * `{:data, request_ref, binary}` - returned when the server replied with\n    a chunk of response body (as a binary). The request shouldn't be considered done\n    when a piece of body is received because multiple chunks could be received. The\n    request is done when the `:done` response is returned.\n\n  * `{:done, request_ref}` - returned when the server signaled the request\n    as done. When this is received, the response body and headers can be considered\n    complete and it can be assumed that no more responses will be received for this\n    request. This means that for example, you can stop holding on to the request ref\n    for this request.\n\n  * `{:error, request_ref, reason}` - returned when there is an error that\n    only affects the request and not the whole connection. For example, if the\n    server sends bad data on a given request, that request will be closed and an error\n    for that request will be returned among the responses, but the connection will\n    remain alive and well.\n\n  * `{:pong, request_ref}` - returned when a server replies to a ping\n    request sent by the client. This response type is HTTP/2-specific\n    and will never be returned by an HTTP/1 connection. See `Mint.HTTP2.ping/2`\n    for more information.\n\n  * `{:push_promise, request_ref, promised_request_ref, headers}` - returned when\n    the server sends a server push to the client. This response type is HTTP/2 specific\n    and will never be returned by an HTTP/1 connection. See `Mint.HTTP2` for more\n    information on server pushes.","ref":"Mint.HTTP.html#stream/2-responses"},{"type":"function","title":"Examples - Mint.HTTP.stream/2","doc":"Let's assume we have a function called `receive_next_and_stream/1` that takes\na connection and then receives the next message, calls `stream/2` with that message\nas an argument, and then returns the result of `stream/2`:\n\n    defp receive_next_and_stream(conn) do\n      receive do\n        message -> Mint.HTTP.stream(conn, message)\n      end\n    end\n\nNow, we can see an example of a workflow involving `stream/2`.\n\n    {:ok, conn, request_ref} = Mint.HTTP.request(conn, \"GET\", \"/\", _headers = [])\n\n    {:ok, conn, responses} = receive_next_and_stream(conn)\n    responses\n    #=> [{:status, ^request_ref, 200}]\n\n    {:ok, conn, responses} = receive_next_and_stream(conn)\n    responses\n    #=> [{:headers, ^request_ref, [{\"Content-Type\", \"application/json\"}]},\n    #=>  {:data, ^request_ref, \"{\"}]\n\n    {:ok, conn, responses} = receive_next_and_stream(conn)\n    responses\n    #=> [{:data, ^request_ref, \"}\"}, {:done, ^request_ref}]","ref":"Mint.HTTP.html#stream/2-examples"},{"type":"function","title":"Mint.HTTP.stream_request_body/3","doc":"Streams a chunk of the request body on the connection or signals the end of the body.\n\nIf a request is opened (through `request/5`) with the body as `:stream`, then the\nbody can be streamed through this function. The function takes a `conn`, a\n`request_ref` returned by `request/5` to identify the request to stream the body for,\nand a chunk of body to stream. The value of chunk can be:\n\n  * iodata - a chunk of iodata is transmitted to the server as part of the body\n    of the request. If the chunk is empty, in HTTP/1 it's a no-op, while in HTTP/2\n    a `DATA` frame will be sent.\n\n  * `:eof` - signals the end of the streaming of the request body for the given\n    request. Usually the server won't send any reply until this is sent.\n\n  * `{:eof, trailer_headers}` - sends **trailer headers** and signals the end\n    of the streaming of the request body for the given request. This behaves the\n    same way as `:eof` but first sends the trailer headers. See the\n    [*Trailer headers*](#module-trailer-headers) section below.\n\nThis function always returns an updated connection to be stored over the old connection.\n\nFor information about transfer encoding and content length in HTTP/1, see\n`Mint.HTTP1.stream_request_body/3`.","ref":"Mint.HTTP.html#stream_request_body/3"},{"type":"function","title":"Trailer headers - Mint.HTTP.stream_request_body/3","doc":"HTTP trailer headers can be sent after the body of a request. trailer headers are described\n[in RFC 9110](https://www.rfc-editor.org/rfc/rfc9110#section-6.5).\n\nThe behaviour is slightly different for HTTP/1 and HTTP/2:\n\n  * In HTTP/1, trailer headers are only supported if the transfer encoding is set to\n    `chunked`. See `Mint.HTTP1.stream_request_body/3` for more information on chunked\n    transfer encoding.\n\n  * In HTTP/2, trailer headers behave like normal headers. You don't need to care\n    about the transfer encoding.","ref":"Mint.HTTP.html#stream_request_body/3-trailer-headers"},{"type":"function","title":"The `trailer` header - Mint.HTTP.stream_request_body/3","doc":"As specified in [section 4.4 of RFC 7230](https://tools.ietf.org/html/rfc7230#section-4.4),\nin HTTP/1 you need to specify which headers you're going to send as traoler\nheaders using the `trailer` header. The `trailer` header applies to both HTTP/1\nand HTTP/2. See the examples below for more information.","ref":"Mint.HTTP.html#stream_request_body/3-the-trailer-header"},{"type":"function","title":"The `te` header - Mint.HTTP.stream_request_body/3","doc":"As specified in  [section 4.3 of RFC 7230](https://tools.ietf.org/html/rfc7230#section-4.3),\nthe `te` (or `TE`) header is used to specify which transfer-encodings the client\nis willing to accept (besides `chunked`). Mint supports decoding of trailer headers,\nbut if you want to notify the server that you are accepting trailer headers,\nuse the `trailers` value in the `te` header. For example:\n\n    Mint.HTTP.request(conn, \"GET\", \"/\", [{\"te\", \"trailers\"}], \"some body\")\n\nNote that the `te` header can also be used to communicate which encodings you\nsupport to the server.","ref":"Mint.HTTP.html#stream_request_body/3-the-te-header"},{"type":"function","title":"Examples - Mint.HTTP.stream_request_body/3","doc":"Let's see an example of streaming an empty JSON object (`{}`) by streaming one curly\nbrace at a time.\n\n    headers = [{\"content-type\", \"application/json\"}, {\"content-length\", \"2\"}]\n    {:ok, conn, request_ref} = Mint.HTTP.request(conn, \"POST\", \"/\", headers, :stream)\n    {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, \"{\")\n    {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, \"}\")\n    {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, :eof)\n\nHere's an example of sending trailer headers:\n\n    headers = [{\"content-type\", \"application/json\"}, {\"trailer\", \"my-trailer, x-expires\"}]\n    {:ok, conn, request_ref} = Mint.HTTP.request(conn, \"POST\", \"/\", headers, :stream)\n\n    {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, \"{}\")\n\n    trailer_headers = [{\"my-trailer\", \"xxx\"}, {\"x-expires\", \"10 days\"}]\n    {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, {:eof, trailer_headers})","ref":"Mint.HTTP.html#stream_request_body/3-examples"},{"type":"opaque","title":"Mint.HTTP.t/0","doc":"","ref":"Mint.HTTP.html#t:t/0"},{"type":"module","title":"Mint.HTTP1","doc":"Process-less HTTP/1.1 client connection.\n\nThis module provides a data structure that represents an HTTP/1 or HTTP/1.1 connection to\na given server. The connection is represented as an opaque struct `%Mint.HTTP1{}`.\nThe connection is a data structure and is not backed by a process, and all the\nconnection handling happens in the process that creates the struct.\n\nThis module and data structure work exactly like the ones described in the `Mint`\nmodule, with the exception that `Mint.HTTP1` specifically deals with HTTP/1 and HTTP/1.1 while\n`Mint` deals seamlessly with HTTP/1, HTTP/1.1, and HTTP/2. For more information on\nhow to use the data structure and client architecture, see `Mint`.","ref":"Mint.HTTP1.html"},{"type":"function","title":"Mint.HTTP1.close/1","doc":"See `Mint.HTTP.close/1`.","ref":"Mint.HTTP1.html#close/1"},{"type":"function","title":"Mint.HTTP1.connect/4","doc":"Same as `Mint.HTTP.connect/4`, but forces an HTTP/1 or HTTP/1.1 connection.\n\nThis function doesn't support proxying.","ref":"Mint.HTTP1.html#connect/4"},{"type":"function","title":"Additional Options - Mint.HTTP1.connect/4","doc":"* `:case_sensitive_headers` - (boolean) if set to `true` the case of the supplied\n     headers in requests will be preserved. The default is to lowercase the headers\n     because HTTP/1.1 header names are case-insensitive. *Available since v1.6.0*.\n  * `:skip_target_validation` - (boolean) if set to `true` the target of a request\n     will not be validated. You might want this if you deal with non standard-\n     conforming URIs but need to preserve them. The default is to validate the request\n     target. *Available since v1.7.0*.","ref":"Mint.HTTP1.html#connect/4-additional-options"},{"type":"function","title":"Mint.HTTP1.controlling_process/2","doc":"See `Mint.HTTP.controlling_process/2`.","ref":"Mint.HTTP1.html#controlling_process/2"},{"type":"function","title":"Mint.HTTP1.delete_private/2","doc":"See `Mint.HTTP.delete_private/2`.","ref":"Mint.HTTP1.html#delete_private/2"},{"type":"function","title":"Mint.HTTP1.get_private/3","doc":"See `Mint.HTTP.get_private/3`.","ref":"Mint.HTTP1.html#get_private/3"},{"type":"function","title":"Mint.HTTP1.get_proxy_headers/1","doc":"See `Mint.HTTP.get_proxy_headers/1`.","ref":"Mint.HTTP1.html#get_proxy_headers/1"},{"type":"function","title":"Mint.HTTP1.get_socket/1","doc":"See `Mint.HTTP.get_socket/1`.","ref":"Mint.HTTP1.html#get_socket/1"},{"type":"function","title":"Mint.HTTP1.open?/2","doc":"See `Mint.HTTP.open?/1`.","ref":"Mint.HTTP1.html#open?/2"},{"type":"function","title":"Mint.HTTP1.open_request_count/1","doc":"See `Mint.HTTP.open_request_count/1`.\n\nIn HTTP/1, the number of open requests is the number of pipelined requests.","ref":"Mint.HTTP1.html#open_request_count/1"},{"type":"function","title":"Mint.HTTP1.put_log/2","doc":"See `Mint.HTTP.put_log/2`.","ref":"Mint.HTTP1.html#put_log/2"},{"type":"function","title":"Mint.HTTP1.put_private/3","doc":"See `Mint.HTTP.put_private/3`.","ref":"Mint.HTTP1.html#put_private/3"},{"type":"function","title":"Mint.HTTP1.recv/3","doc":"See `Mint.HTTP.recv/3`.","ref":"Mint.HTTP1.html#recv/3"},{"type":"function","title":"Mint.HTTP1.request/5","doc":"See `Mint.HTTP.request/5`.\n\nIn HTTP/1 and HTTP/1.1, you can't open a new request if you're streaming the body of\nanother request. If you try, an error will be returned.","ref":"Mint.HTTP1.html#request/5"},{"type":"function","title":"Mint.HTTP1.set_mode/2","doc":"See `Mint.HTTP.set_mode/2`.","ref":"Mint.HTTP1.html#set_mode/2"},{"type":"function","title":"Mint.HTTP1.stream/2","doc":"See `Mint.HTTP.stream/2`.","ref":"Mint.HTTP1.html#stream/2"},{"type":"function","title":"Mint.HTTP1.stream_request_body/3","doc":"See `Mint.HTTP.stream_request_body/3`.\n\nIn HTTP/1, sending an empty chunk is a no-op.","ref":"Mint.HTTP1.html#stream_request_body/3"},{"type":"function","title":"Transfer encoding and content length - Mint.HTTP1.stream_request_body/3","doc":"When streaming the request body, Mint cannot send a precalculated `content-length`\nrequest header because it doesn't know the body that you'll stream. However, Mint\nwill transparently handle the presence of a `content-length` header using this logic:\n\n  * if you specifically set a `content-length` header, then transfer encoding and\n    making sure the content length is correct for what you'll stream is up to you.\n\n  * if you specifically set the transfer encoding (`transfer-encoding` header)\n    to `chunked`, then it's up to you to\n    [properly encode chunks](https://en.wikipedia.org/wiki/Chunked_transfer_encoding).\n\n  * if you don't set the transfer encoding to `chunked` and don't provide a\n    `content-length` header, Mint will do implicit `chunked` transfer encoding\n    (setting the `transfer-encoding` header appropriately) and will take care\n    of properly encoding the chunks.","ref":"Mint.HTTP1.html#stream_request_body/3-transfer-encoding-and-content-length"},{"type":"type","title":"Mint.HTTP1.error_reason/0","doc":"An HTTP/1-specific error reason.\n\nThe values can be:\n\n  * `:closed` - when you try to make a request or stream a body chunk but the connection\n    is closed.\n\n  * `:request_body_is_streaming` - when you call `request/5` to send a new\n    request but another request is already streaming.\n\n  * `{:unexpected_data, data}` - when unexpected data is received from the server.\n\n  * `:invalid_status_line` - when the HTTP/1 status line is invalid.\n\n  * `{:invalid_request_target, target}` - when the request target is invalid.\n\n  * `:invalid_header` - when headers can't be parsed correctly.\n\n  * `{:invalid_header_name, name}` - when a header name is invalid.\n\n  * `{:invalid_header_value, name, value}` - when a header value is invalid. `name`\n    is the name of the header and `value` is the invalid value.\n\n  * `:invalid_chunk_size` - when the chunk size is invalid.\n\n  * `:missing_crlf_after_chunk` - when the CRLF after a chunk is missing.\n\n  * `:invalid_trailer_header` - when trailer headers can't be parsed.\n\n  * `:more_than_one_content_length_header` - when more than one `content-length`\n    headers are in the response.\n\n  * `:transfer_encoding_and_content_length` - when both the `content-length` as well\n    as the `transfer-encoding` headers are in the response.\n\n  * `{:invalid_content_length_header, value}` - when the value of the `content-length`\n    header is invalid, that is, is not an non-negative integer.\n\n  * `:empty_token_list` - when a header that is supposed to contain a list of tokens\n    (such as the `connection` header) doesn't contain any.\n\n  * `{:invalid_token_list, string}` - when a header that is supposed to contain a list\n    of tokens (such as the `connection` header) contains a malformed list of tokens.\n\n  * `:trailing_headers_but_not_chunked_encoding` - when you try to send trailer\n    headers through `stream_request_body/3` but the transfer encoding of the request\n    was not `chunked`.","ref":"Mint.HTTP1.html#t:error_reason/0"},{"type":"opaque","title":"Mint.HTTP1.t/0","doc":"A Mint HTTP/1 connection struct.\n\nThe struct's fields are private.","ref":"Mint.HTTP1.html#t:t/0"},{"type":"module","title":"Mint.HTTP2","doc":"Process-less HTTP/2 client connection.\n\nThis module provides a data structure that represents an HTTP/2 connection to\na given server. The connection is represented as an opaque struct `%Mint.HTTP2{}`.\nThe connection is a data structure and is not backed by a process, and all the\nconnection handling happens in the process that creates the struct.\n\nThis module and data structure work exactly like the ones described in the `Mint.HTTP`\nmodule, with the exception that `Mint.HTTP2` specifically deals with HTTP/2 while\n`Mint.HTTP` deals seamlessly with HTTP/1.1 and HTTP/2. For more information on\nhow to use the data structure and client architecture, see `Mint.HTTP`.","ref":"Mint.HTTP2.html"},{"type":"module","title":"HTTP/2 Streams and Requests - Mint.HTTP2","doc":"HTTP/2 introduces the concept of **streams**. A stream is an isolated conversation\nbetween the client and the server. Each stream is unique and identified by a unique\n**stream ID**, which means that there's no order when data comes on different streams\nsince they can be identified uniquely. A stream closely corresponds to a request, so\nin this documentation and client we will mostly refer to streams as \"requests\".\nWe mentioned data on streams can come in arbitrary order, and streams are requests,\nso the practical effect of this is that performing request A and then request B\ndoes not mean that the response to request A will come before the response to request B.\nThis is why we identify each request with a unique reference returned by `request/5`.\nSee `request/5` for more information.","ref":"Mint.HTTP2.html#module-http-2-streams-and-requests"},{"type":"module","title":"Closed Connection - Mint.HTTP2","doc":"In HTTP/2, the connection can either be open, closed, or only closed for writing.\nWhen a connection is closed for writing, the client cannot send requests or stream\nbody chunks, but it can still read data that the server might be sending. When the\nconnection gets closed on the writing side, a `:server_closed_connection` error is\nreturned. `{:error, request_ref, error}` is returned for requests that haven't been\nprocessed by the server, with the reason of `error` being `:unprocessed`.\nThese requests are safe to retry.","ref":"Mint.HTTP2.html#module-closed-connection"},{"type":"module","title":"HTTP/2 Settings - Mint.HTTP2","doc":"HTTP/2 supports settings negotiation between servers and clients. The server advertises\nits settings to the client and the client advertises its settings to the server. A peer\n(server or client) has to acknowledge the settings advertised by the other peer before\nthose settings come into action (that's why it's called a negotiation).\n\nA first settings negotiation happens right when the connection starts.\nServers and clients can renegotiate settings at any time during the life of the\nconnection.\n\nMint users don't need to care about settings acknowledgements directly since they're\nhandled transparently by `stream/2`.\n\nTo retrieve the server settings, you can use `get_server_setting/2`. Doing so is often\nuseful to be able to tune your requests based on the server settings.\n\nTo communicate client settings to the server, use `put_settings/2` or pass them when\nstarting up a connection with `connect/4`. Note that the server needs to acknowledge\nthe settings sent through `put_setting/2` before those settings come into effect. The\nserver ack is processed transparently by `stream/2`, but this means that if you change\na setting through `put_settings/2` and try to retrieve the value of that setting right\nafter with `get_client_setting/2`, you'll likely get the old value of that setting. Once\nthe server acknowledges the new settings, the updated value will be returned by\n`get_client_setting/2`.","ref":"Mint.HTTP2.html#module-http-2-settings"},{"type":"module","title":"Server Push - Mint.HTTP2","doc":"HTTP/2 supports [server push](https://en.wikipedia.org/wiki/HTTP/2_Server_Push), which\nis a way for a server to send a response to a client without the client needing to make\nthe corresponding request. The server sends a `:push_promise` response to a normal request:\nthis creates a new request reference. Then, the server sends normal responses for the newly\ncreated request reference.\n\nLet's see an example. We will ask the server for `\"/index.html\"` and the server will\nsend us a push promise for `\"/style.css\"`.\n\n    {:ok, conn} = Mint.HTTP2.connect(:https, \"example.com\", 443)\n    {:ok, conn, request_ref} = Mint.HTTP2.request(conn, \"GET\", \"/index.html\", _headers = [], _body = \"\")\n\n    next_message =\n      receive do\n        msg -> msg\n      end\n\n    {:ok, conn, responses} = Mint.HTTP2.stream(conn, next_message)\n\n    [\n      {:push_promise, ^request_ref, promised_request_ref, promised_headers},\n      {:status, ^request_ref, 200},\n      {:headers, ^request_ref, []},\n      {:data, ^request_ref, \" ...\"},\n      {:done, ^request_ref}\n    ] = responses\n\n    promised_headers\n    #=> [{\":method\", \"GET\"}, {\":path\", \"/style.css\"}]\n\nAs you can see in the example above, when the server sends a push promise then a\n`:push_promise` response is returned as a response to a request. The `:push_promise`\nresponse contains a `promised_request_ref` and some `promised_headers`. The\n`promised_request_ref` is the new request ref that pushed responses will be tagged with.\n`promised_headers` are headers that tell the client *what request* the promised response\nwill respond to. The idea is that the server tells the client a request the client will\nwant to make and then preemptively sends a response for that request. Promised headers\nwill always include `:method`, `:path`, and `:authority`.\n\n    next_message =\n      receive do\n        msg -> msg\n      end\n\n    {:ok, conn, responses} = Mint.HTTP2.stream(conn, next_message)\n\n    [\n      {:status, ^promised_request_ref, 200},\n      {:headers, ^promised_request_ref, []},\n      {:data, ^promised_request_ref, \"body { ... }\"},\n      {:done, ^promised_request_ref}\n    ]\n\nThe response to a promised request is like a response to any normal request.\n\n> #### Disabling Server Pushes {: .tip}\n>\n> HTTP/2 exposes a boolean setting for enabling or disabling server pushes with `:enable_push`.\n> You can pass this option when connecting or in `put_settings/2`. By default server push\n> is enabled.","ref":"Mint.HTTP2.html#module-server-push"},{"type":"function","title":"Mint.HTTP2.cancel_request/2","doc":"Cancels an in-flight request.\n\nThis function is HTTP/2 specific. It cancels an in-flight request. The server could have\nalready sent responses for the request you want to cancel: those responses will be parsed\nby the connection but not returned to the user. No more responses\nto a request will be returned after you call `cancel_request/2` on that request.\n\nIf there's no error in canceling the request, `{:ok, conn}` is returned where `conn` is\nthe updated connection. If there's an error, `{:error, conn, reason}` is returned where\n`conn` is the updated connection and `reason` is the error reason.","ref":"Mint.HTTP2.html#cancel_request/2"},{"type":"function","title":"Examples - Mint.HTTP2.cancel_request/2","doc":"{:ok, conn, ref} = Mint.HTTP2.request(conn, \"GET\", \"/\", _headers = [])\n    {:ok, conn} = Mint.HTTP2.cancel_request(conn, ref)","ref":"Mint.HTTP2.html#cancel_request/2-examples"},{"type":"function","title":"Mint.HTTP2.close/1","doc":"See `Mint.HTTP.close/1`.","ref":"Mint.HTTP2.html#close/1"},{"type":"function","title":"Mint.HTTP2.connect/4","doc":"Same as `Mint.HTTP.connect/4`, but forces a HTTP/2 connection.","ref":"Mint.HTTP2.html#connect/4"},{"type":"function","title":"Mint.HTTP2.controlling_process/2","doc":"See `Mint.HTTP.controlling_process/2`.","ref":"Mint.HTTP2.html#controlling_process/2"},{"type":"function","title":"Mint.HTTP2.delete_private/2","doc":"See `Mint.HTTP.delete_private/2`.","ref":"Mint.HTTP2.html#delete_private/2"},{"type":"function","title":"Mint.HTTP2.get_client_setting/2","doc":"Gets the value of the given HTTP/2 client setting.\n\nThis function returns the value of the given HTTP/2 setting that the client\nadvertised to the server. Client settings can be advertised through `put_settings/2`\nor when starting up a connection.\n\nClient settings have to be acknowledged by the server before coming into effect.\n\nThis function is HTTP/2 specific. For more information on HTTP/2 settings, see\n[the related section in the RFC](https://httpwg.org/specs/rfc7540.html#SettingValues).\n\nSee the \"HTTP/2 settings\" section in the module documentation for more information.","ref":"Mint.HTTP2.html#get_client_setting/2"},{"type":"function","title":"Supported settings - Mint.HTTP2.get_client_setting/2","doc":"The possible settings that can be retrieved are described in `t:setting/0`.\nAny other atom passed as `name` will raise an error.","ref":"Mint.HTTP2.html#get_client_setting/2-supported-settings"},{"type":"function","title":"Examples - Mint.HTTP2.get_client_setting/2","doc":"Mint.HTTP2.get_client_setting(conn, :max_concurrent_streams)\n    #=> 500","ref":"Mint.HTTP2.html#get_client_setting/2-examples"},{"type":"function","title":"Mint.HTTP2.get_private/3","doc":"See `Mint.HTTP.get_private/3`.","ref":"Mint.HTTP2.html#get_private/3"},{"type":"function","title":"Mint.HTTP2.get_proxy_headers/1","doc":"See `Mint.HTTP.get_proxy_headers/1`.","ref":"Mint.HTTP2.html#get_proxy_headers/1"},{"type":"function","title":"Mint.HTTP2.get_server_setting/2","doc":"Gets the value of the given HTTP/2 server settings.\n\nThis function returns the value of the given HTTP/2 setting that the server\nadvertised to the client. This function is HTTP/2 specific.\nFor more information on HTTP/2 settings, see [the related section in\nthe RFC](https://httpwg.org/specs/rfc7540.html#SettingValues).\n\nSee the \"HTTP/2 settings\" section in the module documentation for more information.","ref":"Mint.HTTP2.html#get_server_setting/2"},{"type":"function","title":"Supported settings - Mint.HTTP2.get_server_setting/2","doc":"The possible settings that can be retrieved are described in `t:setting/0`.\nAny other atom passed as `name` will raise an error.","ref":"Mint.HTTP2.html#get_server_setting/2-supported-settings"},{"type":"function","title":"Examples - Mint.HTTP2.get_server_setting/2","doc":"Mint.HTTP2.get_server_setting(conn, :max_concurrent_streams)\n    #=> 500","ref":"Mint.HTTP2.html#get_server_setting/2-examples"},{"type":"function","title":"Mint.HTTP2.get_socket/1","doc":"See `Mint.HTTP.get_socket/1`.","ref":"Mint.HTTP2.html#get_socket/1"},{"type":"function","title":"Mint.HTTP2.get_window_size/2","doc":"Returns the window size of the connection or of a single request.\n\nThis function is HTTP/2 specific. It returns the window size of\neither the connection if `connection_or_request` is `:connection` or of a single\nrequest if `connection_or_request` is `{:request, request_ref}`.\n\nUse this function to check the window size of the connection before sending a\nfull request. Also use this function to check the window size of both the\nconnection and of a request if you want to stream body chunks on that request.\n\nFor more information on flow control and window sizes in HTTP/2, see the section\nbelow.","ref":"Mint.HTTP2.html#get_window_size/2"},{"type":"function","title":"HTTP/2 Flow Control - Mint.HTTP2.get_window_size/2","doc":"In HTTP/2, flow control is implemented through a\nwindow size. When the client sends data to the server, the window size is decreased\nand the server needs to \"refill\" it on the client side. You don't need to take care of\nthe refilling of the client window as it happens behind the scenes in `stream/2`.\n\nA window size is kept for the entire connection and all requests affect this window\nsize. A window size is also kept per request.\n\nThe only thing that affects the window size is the body of a request, regardless of\nif it's a full request sent with `request/5` or body chunks sent through\n`stream_request_body/3`. That means that if we make a request with a body that is\nfive bytes long, like `\"hello\"`, the window size of the connection and the window size\nof that particular request will decrease by five bytes.\n\nIf we use all the window size before the server refills it, functions like\n`request/5` will return an error.","ref":"Mint.HTTP2.html#get_window_size/2-http-2-flow-control"},{"type":"function","title":"Examples - Mint.HTTP2.get_window_size/2","doc":"On the connection:\n\n    HTTP2.get_window_size(conn, :connection)\n    #=> 65_536\n\nOn a single streamed request:\n\n    {:ok, conn, request_ref} = HTTP2.request(conn, \"GET\", \"/\", [], :stream)\n    HTTP2.get_window_size(conn, {:request, request_ref})\n    #=> 65_536\n\n    {:ok, conn} = HTTP2.stream_request_body(conn, request_ref, \"hello\")\n    HTTP2.get_window_size(conn, {:request, request_ref})\n    #=> 65_531","ref":"Mint.HTTP2.html#get_window_size/2-examples"},{"type":"function","title":"Mint.HTTP2.open?/2","doc":"See `Mint.HTTP.open?/1`.","ref":"Mint.HTTP2.html#open?/2"},{"type":"function","title":"Mint.HTTP2.open_request_count/1","doc":"See `Mint.HTTP.open_request_count/1`.\n\nIn HTTP/2, the number of open requests is the number of requests **opened by the client**\nthat have not yet received a `:done` response. It's important to note that only\nrequests opened by the client (with `request/5`) count towards the number of open\nrequests, as requests opened from the server with server pushes (see the \"Server push\"\nsection in the module documentation) are not considered open requests. We do this because\nclients might need to know how many open requests there are because the server limits\nthe number of concurrent requests the client can open. To know how many requests the client\ncan open, see `get_server_setting/2` with the `:max_concurrent_streams` setting.","ref":"Mint.HTTP2.html#open_request_count/1"},{"type":"function","title":"Mint.HTTP2.ping/2","doc":"Pings the server.\n\nThis function is specific to HTTP/2 connections. It sends a **ping** request to\nthe server `conn` is connected to. A `{:ok, conn, request_ref}` tuple is returned,\nwhere `conn` is the updated connection and `request_ref` is a unique reference that\nidentifies this ping request. The response to a ping request is returned by `stream/2`\nas a `{:pong, request_ref}` tuple. If there's an error, this function returns\n`{:error, conn, reason}` where `conn` is the updated connection and `reason` is the\nerror reason.\n\n`payload` must be an 8-byte binary with arbitrary content. When the server responds to\na ping request, it will use that same payload. By default, the payload is an 8-byte\nbinary with all bits set to `0`.\n\nPinging can be used to measure the latency with the server and to ensure the connection\nis alive and well.","ref":"Mint.HTTP2.html#ping/2"},{"type":"function","title":"Examples - Mint.HTTP2.ping/2","doc":"{:ok, conn, ref} = Mint.HTTP2.ping(conn)","ref":"Mint.HTTP2.html#ping/2-examples"},{"type":"function","title":"Mint.HTTP2.put_log/2","doc":"See `Mint.HTTP.put_log/2`.","ref":"Mint.HTTP2.html#put_log/2"},{"type":"function","title":"Mint.HTTP2.put_private/3","doc":"See `Mint.HTTP.put_private/3`.","ref":"Mint.HTTP2.html#put_private/3"},{"type":"function","title":"Mint.HTTP2.put_settings/2","doc":"Communicates the given **client settings** to the server.\n\nThis function is HTTP/2-specific.\n\nThis function takes a connection and a keyword list of HTTP/2 settings and sends\nthe values of those settings to the server. The settings won't be effective until\nthe server acknowledges them, which will be handled transparently by `stream/2`.\n\nThis function returns `{:ok, conn}` when sending the settings to the server is\nsuccessful, with `conn` being the updated connection. If there's an error, this\nfunction returns `{:error, conn, reason}` with `conn` being the updated connection\nand `reason` being the reason of the error.","ref":"Mint.HTTP2.html#put_settings/2"},{"type":"function","title":"Supported Settings - Mint.HTTP2.put_settings/2","doc":"See `t:setting/0` for the supported settings. You can see the meaning\nof these settings [in the corresponding section in the HTTP/2\nRFC](https://httpwg.org/specs/rfc7540.html#SettingValues).\n\nSee the \"HTTP/2 settings\" section in the module documentation for more information.","ref":"Mint.HTTP2.html#put_settings/2-supported-settings"},{"type":"function","title":"Examples - Mint.HTTP2.put_settings/2","doc":"{:ok, conn} = Mint.HTTP2.put_settings(conn, max_frame_size: 100)","ref":"Mint.HTTP2.html#put_settings/2-examples"},{"type":"function","title":"Mint.HTTP2.recv/3","doc":"See `Mint.HTTP.recv/3`.","ref":"Mint.HTTP2.html#recv/3"},{"type":"function","title":"Mint.HTTP2.request/5","doc":"See `Mint.HTTP.request/5`.\n\nIn HTTP/2, opening a request means opening a new HTTP/2 stream (see the\nmodule documentation). This means that a request could fail because the\nmaximum number of concurrent streams allowed by the server has been reached.\nIn that case, the error reason `:too_many_concurrent_requests` is returned.\nIf you want to avoid incurring in this error, you can retrieve the value of\nthe maximum number of concurrent streams supported by the server through\n`get_server_setting/2` (passing in the `:max_concurrent_streams` setting name).","ref":"Mint.HTTP2.html#request/5"},{"type":"function","title":"Header list size - Mint.HTTP2.request/5","doc":"In HTTP/2, the server can optionally specify a maximum header list size that\nthe client needs to respect when sending headers. The header list size is calculated\nby summing the length (in bytes) of each header name plus value, plus 32 bytes for\neach header. Note that pseudo-headers (like `:path` or `:method`) count towards\nthis size. If the size is exceeded, an error is returned. To check what the size\nis, use `get_server_setting/2`.","ref":"Mint.HTTP2.html#request/5-header-list-size"},{"type":"function","title":"Request body size - Mint.HTTP2.request/5","doc":"If the request body size will exceed the window size of the HTTP/2 stream created by the\nrequest or the window size of the connection Mint will return a `:exceeds_window_size`\nerror.\n\nTo ensure you do not exceed the window size it is recommended to stream the request\nbody by initially passing `:stream` as the body and sending the body in chunks using\n`stream_request_body/3` and using `get_window_size/2` to get the window size of the\nrequest and connection.","ref":"Mint.HTTP2.html#request/5-request-body-size"},{"type":"function","title":"Mint.HTTP2.set_mode/2","doc":"See `Mint.HTTP.set_mode/2`.","ref":"Mint.HTTP2.html#set_mode/2"},{"type":"function","title":"Mint.HTTP2.stream/2","doc":"See `Mint.HTTP.stream/2`.","ref":"Mint.HTTP2.html#stream/2"},{"type":"function","title":"Mint.HTTP2.stream_request_body/3","doc":"See `Mint.HTTP.stream_request_body/3`.","ref":"Mint.HTTP2.html#stream_request_body/3"},{"type":"type","title":"Mint.HTTP2.error_reason/0","doc":"An HTTP/2-specific error reason.\n\nThe values can be:\n\n  * `:closed` - when you try to make a request or stream a body chunk but the connection\n    is closed.\n\n  * `:closed_for_writing` - when you try to make a request or stream a body chunk but\n    the connection is closed for writing. This means you cannot issue any more requests.\n    See the \"Closed connection\" section in the module documentation for more information.\n\n  * `:too_many_concurrent_requests` - when the maximum number of concurrent requests\n    allowed by the server is reached. To find out what this limit is, use `get_setting/2`\n    with the `:max_concurrent_streams` setting name.\n\n  * `{:max_header_list_size_exceeded, size, max_size}` - when the maximum size of\n    the header list is reached. `size` is the actual value of the header list size,\n    `max_size` is the maximum value allowed. See `get_setting/2` to retrieve the\n    value of the max size.\n\n  * `{:exceeds_window_size, what, window_size}` - when the data you're trying to send\n    exceeds the window size of the connection (if `what` is `:connection`) or of a request\n    (if `what` is `:request`). `window_size` is the allowed window size. See\n    `get_window_size/2`.\n\n  * `{:stream_not_found, stream_id}` - when the given request is not found.\n\n  * `:unknown_request_to_stream` - when you're trying to stream data on an unknown\n    request.\n\n  * `:request_is_not_streaming` - when you try to send data (with `stream_request_body/3`)\n    on a request that is not open for streaming.\n\n  * `:unprocessed` - when a request was closed because it was not processed by the server.\n    When this error is returned, it means that the server hasn't processed the request at all,\n    so it's safe to retry the given request on a different or new connection.\n\n  * `{:server_closed_request, error_code}` - when the server closes the request.\n    `error_code` is the reason why the request was closed.\n\n  * `{:server_closed_connection, reason, debug_data}` - when the server closes the connection\n    gracefully or because of an error. In HTTP/2, this corresponds to a `GOAWAY` frame.\n    `error` is the reason why the connection was closed. `debug_data` is additional debug data.\n\n  * `{:frame_size_error, frame}` - when there's an error with the size of a frame.\n    `frame` is the frame type, such as `:settings` or `:window_update`.\n\n  * `{:protocol_error, debug_data}` - when there's a protocol error.\n    `debug_data` is a string that explains the nature of the error.\n\n  * `{:compression_error, debug_data}` - when there's a header compression error.\n    `debug_data` is a string that explains the nature of the error.\n\n  * `{:flow_control_error, debug_data}` - when there's a flow control error.\n    `debug_data` is a string that explains the nature of the error.","ref":"Mint.HTTP2.html#t:error_reason/0"},{"type":"type","title":"Mint.HTTP2.setting/0","doc":"HTTP/2 setting with its value.\n\nThis type represents both server settings as well as client settings. To retrieve\nserver settings use `get_server_setting/2` and to retrieve client settings use\n`get_client_setting/2`. To send client settings to the server, see `put_settings/2`.\n\nThe supported settings are the following:\n\n  * `:header_table_size` - corresponds to `SETTINGS_HEADER_TABLE_SIZE`.\n\n  * `:enable_push` - corresponds to `SETTINGS_ENABLE_PUSH`. Sets whether\n    push promises are supported. If you don't want to support push promises,\n    use `put_settings/2` to tell the server that your client doesn't want push promises.\n\n  * `:max_concurrent_streams` - corresponds to `SETTINGS_MAX_CONCURRENT_STREAMS`.\n    Tells what is the maximum number of streams that the peer sending this (client or server)\n    supports. As mentioned in the module documentation, HTTP/2 streams are equivalent to\n    requests, so knowing the maximum number of streams that the server supports can be useful\n    to know how many concurrent requests can be open at any time. Use `get_server_setting/2`\n    to find out how many concurrent streams the server supports.\n\n  * `:initial_window_size` -  corresponds to `SETTINGS_INITIAL_WINDOW_SIZE`.\n    Tells what is the value of the initial HTTP/2 window size for the peer\n    that sends this setting.\n\n  * `:max_frame_size` - corresponds to `SETTINGS_MAX_FRAME_SIZE`. Tells what is the\n    maximum size of an HTTP/2 frame for the peer that sends this setting.\n\n  * `:max_header_list_size` - corresponds to `SETTINGS_MAX_HEADER_LIST_SIZE`.\n\n  * `:enable_connect_protocol` - corresponds to `SETTINGS_ENABLE_CONNECT_PROTOCOL`.\n    Sets whether the client may invoke the extended connect protocol which is used to\n    bootstrap WebSocket connections.","ref":"Mint.HTTP2.html#t:setting/0"},{"type":"type","title":"Mint.HTTP2.settings/0","doc":"HTTP/2 settings.\n\nSee `t:setting/0`.","ref":"Mint.HTTP2.html#t:settings/0"},{"type":"opaque","title":"Mint.HTTP2.t/0","doc":"A Mint HTTP/2 connection struct.\n\nThe struct's fields are private.","ref":"Mint.HTTP2.html#t:t/0"},{"type":"module","title":"Mint.Types","doc":"HTTP-related types.","ref":"Mint.Types.html"},{"type":"type","title":"Mint.Types.address/0","doc":"A hostname, IP address, Unix domain socket path, `:loopback`, or any\nother term representing an internet address.","ref":"Mint.Types.html#t:address/0"},{"type":"type","title":"Mint.Types.error/0","doc":"An error reason.","ref":"Mint.Types.html#t:error/0"},{"type":"type","title":"Mint.Types.headers/0","doc":"HTTP headers.\n\nHeaders are sent and received as lists of two-element tuples containing two strings,\nthe header name and header value.","ref":"Mint.Types.html#t:headers/0"},{"type":"type","title":"Mint.Types.http2_response/0","doc":"An HTTP/2-specific response to a request.\n\nThis type of response is only returned on HTTP/2 connections. See `t:response/0` for\nmore response types.","ref":"Mint.Types.html#t:http2_response/0"},{"type":"type","title":"Mint.Types.request_ref/0","doc":"A request reference that uniquely identifies a request.\n\nResponses for a request are always tagged with a request reference so that you\ncan connect each response to the right request. Also see `Mint.HTTP.request/5`.","ref":"Mint.Types.html#t:request_ref/0"},{"type":"type","title":"Mint.Types.response/0","doc":"A response to a request.\n\nTerms of this type are returned as responses to requests. See `Mint.HTTP.stream/2`\nfor more information.","ref":"Mint.Types.html#t:response/0"},{"type":"type","title":"Mint.Types.scheme/0","doc":"The scheme to use when connecting to an HTTP server.","ref":"Mint.Types.html#t:scheme/0"},{"type":"type","title":"Mint.Types.socket/0","doc":"The connection socket.","ref":"Mint.Types.html#t:socket/0"},{"type":"type","title":"Mint.Types.status/0","doc":"An HTTP status code.\n\nThe type for an HTTP is a generic non-negative integer since we don't formally check that\nthe response code is in the \"common\" range (`200..599`).","ref":"Mint.Types.html#t:status/0"},{"type":"exception","title":"Mint.HTTPError","doc":"An HTTP error.\n\nThis exception struct is used to represent HTTP errors of all sorts and for\nboth HTTP/1 and HTTP/2.\n\nA `Mint.HTTPError` struct is an exception, so it can be raised as any\nother exception.","ref":"Mint.HTTPError.html"},{"type":"exception","title":"Struct - Mint.HTTPError","doc":"The `Mint.HTTPError` struct is opaque, that is, not all of its fields are public.\nThe list of public fields is:\n\n  * `:reason` - the error reason. Can be one of:\n\n    * a term of type `t:Mint.HTTP1.error_reason/0`. See its documentation for\n      more information.\n\n    * a term of type `t:Mint.HTTP2.error_reason/0`. See its documentation for\n      more information.\n\n    * `{:proxy, reason}`, which is used when an HTTP error happens when connecting\n      to a tunnel proxy. `reason` can be:\n\n      * `:tunnel_timeout` - when the tunnel times out.\n\n      * `{:unexpected_status, status}` - when the proxy returns an unexpected\n        status `status`.\n\n      * `{:unexpected_trailing_responses, responses}` - when the proxy returns\n        unexpected responses (`responses`).","ref":"Mint.HTTPError.html#module-struct"},{"type":"exception","title":"Message representation - Mint.HTTPError","doc":"If you want to convert an error reason to a human-friendly message (for example\nfor using in logs), you can use `Exception.message/1`:\n\n    iex> {:error, %Mint.HTTPError{} = error} = Mint.HTTP.connect(:http, \"bad-response.com\", 80)\n    iex> Exception.message(error)\n    \"the response contains two or more Content-Length headers\"","ref":"Mint.HTTPError.html#module-message-representation"},{"type":"function","title":"Mint.HTTPError.message/1","doc":"","ref":"Mint.HTTPError.html#message/1"},{"type":"type","title":"Mint.HTTPError.proxy_reason/0","doc":"","ref":"Mint.HTTPError.html#t:proxy_reason/0"},{"type":"type","title":"Mint.HTTPError.t/0","doc":"","ref":"Mint.HTTPError.html#t:t/0"},{"type":"exception","title":"Mint.TransportError","doc":"Represents an error with the transport used by an HTTP connection.\n\nA `Mint.TransportError` struct is an exception, so it can be raised as any\nother exception.","ref":"Mint.TransportError.html"},{"type":"exception","title":"Struct fields - Mint.TransportError","doc":"This exception represents an error with the transport (TCP or SSL) used\nby an HTTP connection. The exception struct itself is opaque, that is,\nnot all fields are public. The following are the public fields:\n\n  * `:reason` - a term representing the error reason. The value of this field\n    can be:\n\n      * `:timeout` - if there's a timeout in interacting with the socket.\n\n      * `:closed` - if the connection has been closed.\n\n      * `:protocol_not_negotiated` - if the ALPN protocol negotiation failed.\n\n      * `{:bad_alpn_protocol, protocol}` - when the ALPN protocol is not\n        one of the supported protocols, which are `http/1.1` and `h2`.\n\n      * `t::inet.posix/0` - if there's any other error with the socket,\n        such as `:econnrefused` or `:nxdomain`.\n\n      * `t::ssl.error_alert/0` - if there's an SSL error.","ref":"Mint.TransportError.html#module-struct-fields"},{"type":"exception","title":"Message representation - Mint.TransportError","doc":"If you want to convert an error reason to a human-friendly message (for example\nfor using in logs), you can use `Exception.message/1`:\n\n    iex> {:error, %Mint.TransportError{} = error} = Mint.HTTP.connect(:http, \"nonexistent\", 80)\n    iex> Exception.message(error)\n    \"non-existing domain\"","ref":"Mint.TransportError.html#module-message-representation"},{"type":"function","title":"Mint.TransportError.message/1","doc":"","ref":"Mint.TransportError.html#message/1"},{"type":"type","title":"Mint.TransportError.t/0","doc":"","ref":"Mint.TransportError.html#t:t/0"},{"type":"extras","title":"Architecture","doc":"# Architecture\n\nMint is an HTTP client with a process-less architecture. Mint provides an API where the HTTP connection is represented by a functional and immutable data structure, a **connection**. Every operation you do on the connection (like sending a request) returns an updated connection. A connection wraps a socket that is in active mode (with `active: :once`). Messages coming from the socket are delivered to the process that created the connection. You can hand those messages to Mint so that they can be parsed into responses.\n\nHaving a process-less architecture makes Mint a powerful and composable HTTP client. The developer has more flexibility compared to a client that forces an interface that includes processes. A Mint connection can be stored inside any kind of process, such as GenServers or GenStage processes.\n\nAnother important feature enabled by the Mint architecture is that a single process can store and manage multiple Mint connections since they are simple data structures. When a message comes from a socket, it's easy to identify which connection it belongs to since that's the only connection that will return a response different from `:unknown` when the messages are parsed. This enables developers to use Mint in fine-tailored ways that suit their needs.","ref":"architecture.html"},{"type":"extras","title":"Pooling - Architecture","doc":"The Mint connection architecture is low level. This means that it doesn't provide things like connection pooling out of the box. This is by design: it's hard to write a general purpose HTTP connection pool that fits all use cases and it's often better to write simple pools that better fit your own use case. In this page, we'll see a few possible uses of Mint including some simple connection pools that you can use as a base for writing your own.","ref":"architecture.html#pooling"},{"type":"extras","title":"Usage examples - Architecture","doc":"Let's see a few example of how to use Mint connections.","ref":"architecture.html#usage-examples"},{"type":"extras","title":"Wrapping a Mint connection in a GenServer - Architecture","doc":"In this example we will look at wrapping a single connection in a GenServer. This architecture can be useful if you only need to issue a few requests to the same host, or if you plan to have many connections spread over just as many processes.\n\nThe way this architecture works is that a GenServer process holds the connection. The connection is started when the `ConnectionProcess` GenServer starts (in the `init/1` callback). When a request is made, the request is sent but the GenServer keeps processing stuff and doesn't directly reply to the process that asked to send the request. The GenServer will only reply to the caller once a response comes from the server. This allows the GenServer to keep sending requests and processing responses while making the request blocking for the caller.\n\nThis asynchronous architecture also makes the GenServer usable from different processes. If you use HTTP/1, requests will appear to be concurrent to the callers of our GenServer, but the GenServer will pipeline the requests. If you use HTTP/2, the requests will be actually concurrent.\nIf you want to avoid pipelining requests you need to manually queue them or reject them in case there's already an ongoing request.\n\nIn this code we don't handle closed connections and failed requests (for brevity). For example, you could handle closed connections by having the GenServer try to reconnect after a backoff time.\n\n```elixir\ndefmodule ConnectionProcess do\n  use GenServer\n\n  require Logger\n\n  defstruct [:conn, requests: %{}]\n\n  def start_link({scheme, host, port}) do\n    GenServer.start_link(__MODULE__, {scheme, host, port})\n  end\n\n  def request(pid, method, path, headers, body) do\n    GenServer.call(pid, {:request, method, path, headers, body})\n  end","ref":"architecture.html#wrapping-a-mint-connection-in-a-genserver"},{"type":"extras","title":"Callbacks - Architecture","doc":"@impl true\n  def init({scheme, host, port}) do\n    case Mint.HTTP.connect(scheme, host, port) do\n      {:ok, conn} ->\n        state = %__MODULE__{conn: conn}\n        {:ok, state}\n\n      {:error, reason} ->\n        {:stop, reason}\n    end\n  end\n\n  @impl true\n  def handle_call({:request, method, path, headers, body}, from, state) do\n    # In both the successful case and the error case, we make sure to update the connection\n    # struct in the state since the connection is an immutable data structure.\n    case Mint.HTTP.request(state.conn, method, path, headers, body) do\n      {:ok, conn, request_ref} ->\n        state = put_in(state.conn, conn)\n        # We store the caller this request belongs to and an empty map as the response.\n        # The map will be filled with status code, headers, and so on.\n        state = put_in(state.requests[request_ref], %{from: from, response: %{}})\n        {:noreply, state}\n\n      {:error, conn, reason} ->\n        state = put_in(state.conn, conn)\n        {:reply, {:error, reason}, state}\n    end\n  end\n\n  @impl true\n  def handle_info(message, state) do\n    # We should handle the error case here as well, but we're omitting it for brevity.\n    case Mint.HTTP.stream(state.conn, message) do\n      :unknown ->\n        _ = Logger.error(fn -> \"Received unknown message: \" <> inspect(message) end)\n        {:noreply, state}\n\n      {:ok, conn, responses} ->\n        state = put_in(state.conn, conn)\n        state = Enum.reduce(responses, state, &process_response/2)\n        {:noreply, state}\n    end\n  end\n\n  defp process_response({:status, request_ref, status}, state) do\n    put_in(state.requests[request_ref].response[:status], status)\n  end\n\n  defp process_response({:headers, request_ref, headers}, state) do\n    put_in(state.requests[request_ref].response[:headers], headers)\n  end\n\n  defp process_response({:data, request_ref, new_data}, state) do\n    update_in(state.requests[request_ref].response[:data], fn data -> (data || \"\") <> new_data end)\n  end\n\n  # When the request is done, we use GenServer.reply/2 to reply to the caller that was\n  # blocked waiting on this request.\n  defp process_response({:done, request_ref}, state) do\n    {%{response: response, from: from}, state} = pop_in(state.requests[request_ref])\n    GenServer.reply(from, {:ok, response})\n    state\n  end\n\n  # A request can also error, but we're not handling the erroneous responses for\n  # brevity.\nend\n```","ref":"architecture.html#callbacks"},{"type":"extras","title":"Decompression","doc":"# Decompression\n\nMany web servers use compression to reduce the size of the payload to speed up delivery to clients, expecting clients to decompress the body of the request. Some of the common compression algorithms used are [gzip], [brotli], [zstd], or no compression at all.\n\nClients may specify acceptable compression algorithms through the [`accept-encoding`][accept-encoding] request header. It's common for clients to supply one or more values in `accept-encoding`, for example `accept-encoding: gzip, br` in the order of preference.\n\nServers will read the `accept-encoding` and `TE` request headers, and respond appropriately indicating which compression is used in the response body through the [`content-encoding`][content-encoding] or [`transfer-encoding`][transfer-encoding] response headers respectively. It's not as common to use multiple compression algorithms, but it is possible: for example, `content-encoding: gzip` or `content-encoding: br, gzip` (meaning it was compressed with `br` first, and then `gzip`).\n\nMint is a low-level client so it doesn't have built-in support for decompression. In this guide we'll explore how to add support for decompression when using Mint.","ref":"decompression.html"},{"type":"extras","title":"Decompressing the response body - Decompression","doc":"Starting with the [architecture example](Architecture.md#content), we're going add some logic to handle a finished request's compressed body. With some compression algorithms, it's possible to decompress body chunks as they come (in a streaming way), but let's look at an example that works for every compression algorithm by decompressing the whole response body when the response is done.\n\nThis is where we start:\n\n```elixir\ndefp process_response({:done, request_ref}, state) do\n  {%{response: response, from: from}, state} = pop_in(state.requests[request_ref])\n  GenServer.reply(from, {:ok, response})\n  state\nend\n```\n\nThis function handles the response back to the blocked process that's waiting for the HTTP response. You'll see that it returns `{:ok, response}` with `response` being a map with `:status`, `:headers`, and `:data` fields.\n\nWe need to attempt to decompress the data if the `content-encoding` header is present. We're going to work with `content-encoding`, but the same applies if compression is used in `transfer-encoding`. Let's add a function that finds all applied compression algorithms.\n\n```elixir\n# Returns a list of found compressions or [] if none found.\ndefp get_content_encoding_header(headers) do\n  headers\n  |> Enum.flat_map(fn {name, value} ->\n    if String.downcase(name, :ascii) == \"content-encoding\" do\n      value\n      |> String.downcase()\n      |> String.split(\",\", trim: true)\n      |> Stream.map(&String.trim/1)\n    else\n      []\n    end\n  end)\n  |> Enum.reverse()\nend\n```\n\nWe use a combination of `Enum.flat_map/2` and `String.split/3` because the values can be comma-separated and spread over multiple headers. Now we should have a list like `[\"gzip\"]`. We reversed the compression algorithms so that we decompress from the last one to the first one. Let's use this in another function that handles the decompression. Thankfully, Erlang ships with built-in support for gzip algorithm.\n\n```elixir\ndefp decompress_data(data, algorithms) do\n  Enum.reduce(algorithms, data, &decompress_with_algorithm/2)\nend\n\ndefp decompress_with_algorithm(gzip, data) when gzip in [\"gzip\", \"x-gzip\"],\n  do: :zlib.gunzip(data)\n\ndefp decompress_with_algorithm(\"identity\", data),\n  do: data\n\ndefp decompress_with_algorithm(algorithm, data),\n  do: raise \"unsupported decompression algorithm: #{inspect(algorithm)}\"\n```\n\nIn case you come across an unsupported algorithm, you might want to log or raise an exception so you can see where you may be lacking support.\n\nNow let's put it together. We can use these new functions when the request is done and pass the result back to the client.\n\n```elixir\ndefp process_response({:done, request_ref}, state) do\n  {%{response: response, from: from}, state} = pop_in(state.requests[request_ref])\n\n  # Handle compression here.\n  compression_algorithms = get_content_encoding_header(response.headers)\n  response = update_in(response.data, &decompress_data(&1, compression_algorithms))\n\n  GenServer.reply(from, {:ok, response})\n\n  state\nend\n```\n\nNow you can decompress responses! Above is a simple approach to a potentially complex response, so there is room for error. For example, this guide does not handle decompression errors or compression through `transfer-encoding` (although the code stays very similar in that case).\n\n\n[gzip]: https://tools.ietf.org/html/rfc1952\n[brotli]: https://tools.ietf.org/html/rfc7932\n[ztsd]: https://tools.ietf.org/html/rfc8478\n[accept-encoding]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Accept-Encoding\n[content-encoding]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Encoding\n[transfer-encoding]: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Transfer-Encoding","ref":"decompression.html#decompressing-the-response-body"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}